﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public enum Theme
{
    MAIN,
    BOSS
}

public class SoundManager : MonoBehaviour
{
    public AudioSource MainThemeSource;     //Drag a reference to the audio source which will play the music.
    public AudioSource BossThemeSource;

    public AudioSource gameOverMusicSource; // TODO: Get this

    public static SoundManager instance = null;     //Allows other scripts to call functions from SoundManager.             
    public float lowPitchRange = .95f;              //The lowest a sound effect will be randomly pitched.
    public float highPitchRange = 1.05f;            //The highest a sound effect will be randomly pitched.
    public static float FadeTime = 3.0f;

    public Theme StartTheme = Theme.MAIN;

    private Dictionary<Theme, AudioSource> ThemeMap;

    private AudioSource CurrentMusicSource;


    void Awake()
    {
        ThemeMap = new Dictionary<Theme, AudioSource>() {
            {Theme.MAIN, MainThemeSource},
            {Theme.BOSS, BossThemeSource},
        };

        MainThemeSource.loop = true;

        //Check if there is already an instance of SoundManager
        if (instance == null)
            //if not, set it to this.
            instance = this;
        //If instance already exists:
        else if (instance != this)
            //Destroy this, this enforces our singleton pattern so there can only be one instance of SoundManager.
            Destroy(gameObject);

        //Set SoundManager to DontDestroyOnLoad so that it won't be destroyed when reloading our scene.
        DontDestroyOnLoad(gameObject);
    }

    public void Start()
    {
        PlayTheme(StartTheme);
    }

    private void Update()
    {
        if (CurrentMusicSource.clip.length <= CurrentMusicSource.time + 3.0f)
        {
            PlayTheme(Theme.MAIN);
        }
    }

    public void PlayMusicSource(AudioSource audio)
    {
        if (CurrentMusicSource != null)
        {
            CurrentMusicSource.Stop();
        }

        CurrentMusicSource = audio;
        CurrentMusicSource.Play();
    }

    public void PlayTheme(Theme theme)
    {
        AudioSource source;
        var exists = ThemeMap.TryGetValue(theme, out source);
        if (exists)
        {
            if (CurrentMusicSource != null && CurrentMusicSource.isPlaying)
            {
                Transition(source, 3.0f);
            }
            else
            {
                PlayMusicSource(source);
            }
        }
    }

    public AudioSource GetCurrentMusicSource()
    {
        return CurrentMusicSource;
    }

    //Used to play single sound clips.
    public void PlaySingle(AudioSource efxSource, AudioClip clip)
    {
        //Set the clip of our efxSource audio source to the clip passed in as a parameter.
        efxSource.clip = clip;

        //Play the clip.
        efxSource.Play();
    }

    //RandomizeSfx chooses randomly between various audio clips and slightly changes their pitch.
    // It uses the provided AudioSource to play the clip
    public void RandomizeSfx(AudioSource efxSource, params AudioClip[] clips)
    {
        //Generate a random number between 0 and the length of our array of clips passed in.
        int randomIndex = Random.Range(0, clips.Length);

        //Choose a random pitch to play back our clip at between our high and low pitch ranges.
        float randomPitch = Random.Range(lowPitchRange, highPitchRange);

        //Set the pitch of the audio source to the randomly chosen pitch.
        efxSource.pitch = randomPitch;

        //Set the clip to the clip at our randomly chosen index.
        // efxSource.clip = clips[randomIndex];

        //Play the clip.
        efxSource.PlayOneShot(clips[randomIndex]);
    }

    public void Transition(AudioSource nextAudio, float fadeTime = 0)
    {
        if (nextAudio.isPlaying)
            return;

        if (fadeTime != 0)
            fadeTime = FadeTime;
        else
            fadeTime = 0.5f;

        StartCoroutine(FadeOut(CurrentMusicSource, fadeTime));

        CurrentMusicSource = nextAudio;

        StartCoroutine(FadeIn(nextAudio, fadeTime));
    }

    private IEnumerator FadeOut(AudioSource audioSource, float fadeTime)
    {
        float startVolume = audioSource.volume;

        while (audioSource.volume > 0)
        {
            audioSource.volume -= startVolume * Time.deltaTime / fadeTime;

            yield return null;
        }

        audioSource.Stop();
        audioSource.volume = startVolume;
    }

    private IEnumerator FadeIn(AudioSource audioSource, float fadeTime)
    {
        float finalVolume = 1;

        if (audioSource.name.Equals("Hell"))
            finalVolume = 0.6f;

        if (!audioSource.isPlaying)
        {
            audioSource.volume = 0;
            audioSource.Play();
        }

        while (audioSource.volume < finalVolume)
        {
            audioSource.volume += finalVolume * Time.deltaTime / fadeTime;

            yield return null;
        }
    }
}